@@ -873,6 +873,10 @@ mergeMriImages <- function (..., bindDim = NULL, padTags = FALSE)
             {
                 xn <- resolveValue(x[[n]], x$.blocks)
                 yn <- resolveValue(y[[n]], y$.blocks)
+                if (is.matrix(xn) && !is.matrix(yn) && all(is.na(yn)))
+                    yn <- matrix(NA, nrow=length(yn), ncol=ncol(xn))
+                if (is.matrix(yn) && !is.matrix(xn) && all(is.na(xn)))
+                    xn <- matrix(NA, nrow=length(xn), ncol=ncol(yn))
                 if (is.matrix(xn) || is.matrix(yn))
                     rbind(xn, yn)
                 else if (is.null(xn) || is.null(yn))
